class Person
{
  int x, y;               // Position
  color col;              // Color

  int endurance;          // Modifier for moving on uneven terrain
  int sociability;        // Modifier for moving to a square with another person in it
  int terrainPreference;  // What type of terrain does this Person prefer?

  boolean disable;
  int homeX, homeY;       // Location of this Person's 'home' (where they first spawned)

  Person(int x_, int y_, int hx_, int hy_)
  {
    x = x_;
    y = y_;
    
    if (tiles[x+y*terrainMap.width].terrainType == 0 || tiles[x+y*terrainMap.width].terrainType == 2) disable = true;

    col = color(255, 200);

    endurance = int( random(0, 100) );  // If endurance is greater than or equal to the flatness of tile, there is a greater probability of moving there
    sociability = int( random(1, 5) );  // If sociability is greater than or equal to the number of people on a given tile, there is a greater probability of moving there
    terrainPreference = int( random(2) );  // 0 = fertility, 1 = mineral wealth (shouldn't ever actually equal 2)

    homeX = hx_;
    homeY = hy_;
  }


  void move()
  {
    // Determine 1D location of current tile
    int loc = x + y*terrainMap.width;
  
    // Set this tile as having been used one more time
    tiles[loc].timesUsed++;
  
    // Call the current tile's 'settle' function. (ie: Check to see if a settlement should be created here.)
    if (tiles[loc].terrainType != 0) tiles[loc].settle();

    // 'Harvest' from current tile
    if (tiles[loc].brightness <= 80) tiles[loc].amountFarmed += tiles[loc].fertility;
    if (tiles[loc].brightness > 80)tiles[loc].amountMined  += tiles[loc].minWealth;
    if (tiles[loc].amountFarmed >= tiles[loc].farmThreshold && tiles[loc].terrainType == 1)  // If this tile is turned into a farm...
    {
      // Create a new farmer
      workers.add( new Worker(x, y, homeX, homeY, x, y, 0) );
      // Disable this person
      disable = true;
      tiles[loc].terrainType = 3;  // ...set it's terrain type to 3 (for farm)...
      return;
    }
    if (tiles[loc].amountMined >= tiles[loc].mineThreshold && tiles[loc].terrainType == 1)  // If this tile is turned into a mine...
    {
      // Create a new farmer
      workers.add( new Worker(x, y, homeX, homeY, x, y, 1) );
      // Disable this person
      disable = true;
      tiles[loc].terrainType = 5;  // ...set it's terrain type to 5 (for mine)...
      return;
    }

    // Create movement probability matrix and fill each entry with baseProb
    int[][] moveProb = new int[3][3];
    for (int i = 0; i < 3; i++) {
      for (int j = 0; j < 3; j++) {
        moveProb[i][j] = 0;
      }
    }

    float totalProb = 0;  // The added total of all 9 probability values

    // Check out all potential tiles and determine probability of moving to each one
    for (int i = 0; i < 3; i++)
    {
      for (int j = 0; j < 3; j++)
      {
        int tileLoc = (x + (i-1)) + (y + (j-1))*terrainMap.width;  // 1D location of the tile being looked at
        tileLoc = constrain(tileLoc, 0, tiles.length-1);
                  
        // Dealing with specific types of terrain
        if (tiles[tileLoc].terrainType == 1)
        {  // If the tile being looked at is land
        
          // Land tiles start with 1 probability always
          moveProb[i][j] = 1;
  
          // If endurance is greater than flatness, add 1 to this tile's probability
          if (endurance >= tiles[tileLoc].flatness) moveProb[i][j]++;
  
          // Deal with terrain preference
          if (terrainPreference == 0)  // If this Person prefers fertility...
          {
            if (tiles[i].fertility >= tiles[tileLoc].minWealth) moveProb[i][j]++;
          } else if (terrainPreference == 1)  // ... or else, if this Person prefers mineral wealth...
          {
            if (tiles[i].minWealth >= tiles[tileLoc].fertility) moveProb[i][j]++;
          }
        }

        // See how many people are in this tile
        int peopleInTile = 0;
        for (int k = 0; k < people.size (); k++)
        {
          Person p = (Person) people.get(k);
          if ((p.x + p.y * width) == tileLoc)
          {
            peopleInTile++;
          }
        }

        // If this person's sociability rating is greater than or equal to the number of people in that tile, increase the probability of moving there.
        if (sociability <= peopleInTile) moveProb[i][j]++;

        // Add current probability to total
        totalProb += moveProb[i][j];

      } // end for j
    }  // end for i

    // Generate a random number from 0 to totalProb
    float r = random(0, totalProb);
    float lowerRange = 0;
    float upperRange = 0;

    for (int i = 0; i < 3; i++)
    {
      for (int j = 0; j < 3; j++)
      {
        upperRange += moveProb[i][j];
        if (r > lowerRange && r <= upperRange)  // If movement is less than current moveProb
        {
          x += (i-1);
          x = constrain(x, 0, width-1);
          y += (j-1);
          y = constrain(y, 0, height-1);
          return;
        }
        else lowerRange = upperRange;
      }
    }

  } //<>//


  void display()
  {
    fill(col);
    rect(x, y, 1, 1);
  }  // display() end
}  // end class
