class Tile
{
  int x, y;         // Position

  int terrainType;    // 0 = water, 1 = land, 2 = mountain, 3 = farm, 4 = settlement, 5 = mines
  float flatness;    // How level the current tile is compared to it's surroundings
  float timesUsed;      // Number of times a Person has been on this tile
  
  color col;           // Color of the tile
  float brightness;    // Average of rgb (used to calculate various other values; see below)

  // For water
  boolean landAdj;     // Is this pixel adjacent to land?
  float depth;         // Affects fishing potential & movement speed (based on brightness)

  // For land
  boolean waterAdj;       // Is this tile adjacent to a water tile?
  boolean mountainAdj;    // Is this tile adjacent to a mountain?
  
  float minWealth;          // Potential for mining (higher brightness = more minWealth)
  int amountMined;          // Total amount that has been mined from this tile.
  int mineThreshold;        // Amount of minerals necessary to be gathered before this tile becomes a mine
  
  int settlementLevel;      // Size of settlement upon tile.
  int storedFood;           // Amount of food stored in this settlement
  int storedMinerals;       // Amount of minerals stored in this settlement
  int spawnThreshold;       // Amount of food needed to be stored before this settlement spawns a new Person.
  int pioneerThreshold;     // Amount of minerals needed to be stored before thie settlement sends out a company of pioneers (by ship or otherwise)
  
  float fertility;          // Potential for agriculture (lower brightness = more fertility)
  int amountFarmed;         // Amount of food that has been gathered from this tile
  int farmThreshold;        // Amount of food necessary to be gathered before this tile becomes a farm

  Tile(int x_, int y_, int terrainType_, color col_)  // add the additional variables once this one is working.
  {
    x = x_;
    y = y_;
    terrainType = terrainType_; 
    col = col_;
    
    brightness = (red(col) + green(col) + blue(col)) / 3;
    
    // If land type is water, establish depth based on brightness
    if (terrainType == 0) {
      depth = map(brightness, 0, waterThreshold, 0, 100);
    }
    
    // If terrain type is land, establish fertility and mineral wealth based on brightness
    if (terrainType == 1) {
      fertility = map(brightness, 62, 98, 100,     0);
      minWealth = map(brightness, 62, 98,   0,   100);
    }
    
    map(flatness, 0, 100, -50, 50);
    // Modify fertility based on levelness/water adjacency (more levelness = more fertility)
    fertility += flatness;
    if (waterAdj == true) fertility += 15;
    fertility = map(fertility, -50, 165, 0, 100);
    
    timesUsed = 0;
    
    // Modify mineral wealth based on levelness/mountain adjancency (less levelness = more mineral wealth)
    minWealth -= flatness;
    if (mountainAdj == true) minWealth += 15;
    minWealth = map(minWealth, -50, 165, 0, 100);
    
    settlementLevel = 0;
    storedFood = 0;
    spawnThreshold = 1000;
    pioneerThreshold = 2000;
    
    amountMined = 0;
    mineThreshold = 650;
    
    amountFarmed = 0;
    farmThreshold = 650;
  }
  
  // Display tiles which should be below people layer
  void belowPeopleDisplay()
  {
    // For coasts
    if (landAdj == true)
    {
      fill(151, 180, 238);
      rect(x, y, 1, 1);
    }
    // For roads
    if (timesUsed > 0)
    {
      fill(150, map(timesUsed, 0, 40, 0, 200));
      rect(x, y, 1, 1);
    }
    // For farms
    if (terrainType == 3)  
    {
      fill(255, 190, 0, 128);
      rect(x, y, 1, 1);
    }
    // For mines
    if (terrainType == 5)
    {
      fill(0, 128);
      rect(x, y, 1, 1);
    }
  }  // display() end
  
  
  // Display tiles which should be above people layer
  void abovePeopleDisplay()
  {
    // For settlements
    if (settlementLevel > 0)  
    {
      fill(map(storedMinerals, 0, pioneerThreshold, 120, 170), 120, map(storedFood, 0, spawnThreshold, 120, 170));
      rect(x, y, 1, 1);
    }

  }
  
  
  void settle()
  {
    IntList storedPersons = new IntList();  // An integer list to store the array locations of Person objects
    // Check how many People are on this tile
    for (int i = 0; i < people.size(); i++)
      {
        Person p = (Person) people.get(i);
        int tloc =   x +   y*terrainMap.width;  // 1D location of this tile
        int ploc = p.x + p.y*terrainMap.width;  // 1D location of the Person being looked at
        if (ploc == tloc)  // If the person being looked at is on this tile...
        {
          storedPersons.append(i);  // ...store the index number of this Person.
        }
      }
      if (storedPersons.size() >= settlementThreshold && settlementLevel == 0)  // If the number of people on this tile is at the settlement threshold...
      {
        println("Settlement increase!");
        // ...and delete a number of people equal to the settlement threshold...
        storedPersons.sortReverse();
        for (int j = 0; j < settlementThreshold; j++)
        {
          int sp = storedPersons.get(j);
          people.remove(sp);
        }
        // Then create a new person at this settlement
        people.add(new Person(x, y, x, y));
        terrainType = 4;    // (settlements have their own terrainType)
        settlementLevel++;  // ...increase the settlement level...
      }
      storedPersons.clear();  // Clear the list of stored people
  }
  
  
  void spawn()
  {
      people.add(new Person(x, y, x, y));
      println("Person was born! Woop there it is!");
      storedFood -= spawnThreshold;
  } // end spawn()
  
  
  void pioneer()
  {
    // Create a new pioneer
    pioneers.add(new Pioneer(x, y));
    storedMinerals -= pioneerThreshold;
    pioneerThreshold += 2000;
  }
  
} // end class
